<?php

/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

/**
 * Description of AbstractAssertionChecking
 *
 * @author Luis Javier Gomez Santana <luis.gomez at rediris.es>
 */

require_once 'AssertionPolicy.php';




/* * *************************************************************************** */
/* * *********************** AbstractAssertionChecking ************************* */
/* * *************************************************************************** */

abstract class AbstractAssertionChecking {

    protected $policy;
    protected $assertion;
    protected $tokenInfo;
    protected $error;
    protected $scope;
    protected $tokenFormat;

    abstract function checkAssertion($assertion);

    public function getTokenInfo() {
        return $this->tokenInfo;
    }

    public function getError() {
        return $this->error;
    }

    public function getPolicyAttr() {
        return $this->policy;
    }

    public function getPolicy($rulesfile, $assertion_type) {
        $xml = simplexml_load_file($rulesfile);
        if (strcmp($xml->getName(), "AssertionList") == 0) {
            foreach ($xml->children() as $policy) {
                if (0 == strcmp($assertion_type, $policy['type'])) {
                    foreach ($policy->children() as $pol) {
                        if (0 == strcmp($this->scope, $pol['scope'])) {
                            $this->tokenFormat = $pol->TokenFormat;
                            foreach ($pol->Policy as $p) {
                                $this->policy[] = new AssertionPolicy($p);
                            }
                        }
                    }
                }
            }
        }
        return $this->policy;
    }

    public function cleanScope($scope) {
        if (strpos($scope, "?") == 0) {
            $res = $scope;
        } else {
            $res = substr($scope, 0, strpos($scope, "?"));
        }
        return $res;
    }

    //lo llama el método checkAssertion de cada clase que extiende de AbstractAssertionChecking
    protected function matchRules() {
        $dev = true;
        if (sizeof($this->policy) > 0) {
            foreach ($this->policy as $pol) {
                $dev = $dev && $pol->checkPolicy($this->assertion);
            }
        } else {
            $dev = false;
        }
        return $dev;
    }

    //lo llama el método checkAssertion de cada clase que extiende de AbstractAssertionChecking
    public function generateTokenInfo() {
        $array_ret = array();
        foreach ($this->tokenFormat->children() as $attribute) {
            $att = trim($attribute, "%");
            if (array_key_exists($att, $this->assertion)) {
                $array_ret[$att] = $this->assertion[$att];
            } else if (strcmp($att, 'scope') == 0) {
                $array_ret['scope'] = $this->scope;
            }
        }
        return $array_ret;
    }

}

/* * *************************************************************************** */
/* * ************************* SIRAssertionChecking **************************** */
/* * *************************************************************************** */

class SIRAssertionChecking extends AbstractAssertionChecking {

    public function __construct($scope, $dir='') {
        $this->error = false;
        $this->scope = parent::cleanScope($scope);
        $this->policy = array();
        $this->tokenFormat = "%sho%";
        $this->tokenInfo = array();
        if ($dir == '') {
            $file = dirname(dirname(__FILE__)) . "/config/policies.xml";
        } else {
            $file = $dir . "policies.xml";
        }
        if (0 == sizeof(parent::getPolicy($file, 'papi'))) {
            error_log("Error extracting policies");
            $this->error = true;
        }
    }

    public function checkAssertion($assertion) {
        $this->assertion = $userAssertion;
        $dev = false;
        if ($this->matchRules() != false) {
            $this->tokenInfo = $this->generateTokenInfo();
            if ($this->tokenInfo != null) {
                $dev = true;
            }
        }
        return $dev;
    }

}

/* * *************************************************************************** */
/* * ************************ SAML2AssertionChecking *************************** */
/* * *************************************************************************** */

class SAML2AssertionChecking extends AbstractAssertionChecking {

    public function __construct($scope, $dir='') {
        $this->error = false;
        $this->scope = parent::cleanScope($scope);
        $this->policy = array();
        $this->tokenFormat = "%urn:mace:dir:attribute-def:eduPersonScopedAffiliation%";
        $this->tokenInfo = array();
        if ($dir == '') {
            $file = dirname(dirname(__FILE__)) . "/config/policies.xml";
        } else {
            $file = $dir . "policies.xml";
        }
        if (0 == sizeof(parent::getPolicy($file, 'saml2'))) {
            error_log("Error extracting policies");
            $this->error = true;
        }
    }

    public function checkAssertion($assertion) {
        if ($this->validateAssertionFormat($assertion)) {
            $this->assertion = array();
            $this->assertion['urn:mace:dir:attribute-def:eduPersonScopedAffiliation'] = 'rediris.es';
            $this->assertion['urn:mace:dir:attribute-def:mail'] = 'luis.gomez@rediris.es';
            $dev = false;
            if ($this->matchRules() != false) {
                $this->tokenInfo = $this->generateTokenInfo();
                if ($this->tokenInfo != NULL) {
                    $dev = true;
                }
            }
        }
        return $dev;
    }

}

$v = new SIRAssertionChecking('scope_de_prueba', '/Users/luija/Sites/html/oauth2lib/oauth_as/config/');
echo '<pre>';
print_r($v);
echo '</pre></hr>';
$v = new SAML2AssertionChecking('scope_de_prueba_SAML', '/Users/luija/Sites/html/oauth2lib/oauth_as/config/');
echo '<pre>';
print_r($v);
echo '</pre>';
?>
